# -*- coding: utf-8 -*-
import weakref
from abc import abstractmethod
from typing import Union

from PyQt5.QtGui import QCursor
from PyQt5.QtWidgets import QMenu, QWidgetAction, QLineEdit

from ..signal_connection import SignalContainer
from ...utils import RoiParameters


class AbstractRoiContextMenu(QMenu):
    @property
    def roi(self):
        return self._roi()

    def __init__(self, roi: 'AbstractROI'):
        QMenu.__init__(self)
        self.value = roi.value
        self._roi = weakref.ref(roi)
        self.__init_menu__()
        self.exec_(QCursor.pos())

    def send(self, signal: Union[SignalContainer, str, tuple]):
        if not self.roi:
            return
        if isinstance(signal, str):
            self.roi.signal_by_key.emit((signal,))
        elif isinstance(signal, tuple):
            self.roi.signal_by_key.emit(signal)
        else:
            self.roi.arbitrary_signal.emit(signal)

    @abstractmethod
    def __init_menu__(self):
        pass


class RoiContextMenu(AbstractRoiContextMenu):
    def __init_menu__(self):
        self.__init_rename_menu__()
        self.addSeparator()
        self.__init_type_menu__()
        self.addSeparator()
        self.__init_fix_menu__()
        self.addSeparator()
        self.__init_select_menu__()
        self.addSeparator()
        self.__init_delete_menu__()

    def __init_fix_menu__(self):
        fix_menu = self.addMenu('Fix/Unfix')
        if not self.value.movable:
            fix_action = fix_menu.addAction('Unfix roi')
            fix_action.triggered.connect(
                lambda: self.send(SignalContainer().segment_unfixed(self.value)))
        else:
            fix_action = fix_menu.addAction('Fix roi')
            fix_action.triggered.connect(
                lambda: self.send(SignalContainer().segment_fixed(self.value)))
        fix_selected = fix_menu.addAction('Fix selected roi')
        fix_selected.triggered.connect(
            lambda: self.send('fix_selected'))
        unfix_selected = fix_menu.addAction('Unfix selected roi')
        unfix_selected.triggered.connect(
            lambda: self.send('unfix_selected'))

        fix_all = fix_menu.addAction('Fix all roi')
        fix_all.triggered.connect(
            lambda: self.send('fix_all'))

        unfix_all = fix_menu.addAction('Unix all roi')
        unfix_all.triggered.connect(
            lambda: self.send('unfix_all'))

    def __init_delete_menu__(self):
        delete_menu = self.addMenu('Delete')
        delete_self = delete_menu.addAction('Delete this roi')
        delete_self.triggered.connect(
            lambda: self.send(SignalContainer().segment_deleted(self.value)))
        delete_selected = delete_menu.addAction('Delete selected')
        delete_selected.triggered.connect(
            lambda: self.send('delete_selected'))

    def __init_select_menu__(self):
        select_menu = self.addMenu('Select')
        select_all = select_menu.addAction('Select all')
        select_all.triggered.connect(
            lambda: self.send('select_all'))
        unselect_all = select_menu.addAction('Unselect all')
        unselect_all.triggered.connect(
            lambda: self.send('unselect_all'))

    def __init_rename_menu__(self):
        rename = self.addMenu('Rename')
        rename_action = QWidgetAction(self)
        line_edit = QLineEdit(self.value.name)
        line_edit.editingFinished.connect(
            lambda: self.roi.send_name(line_edit.text())
        )
        rename_action.setDefaultWidget(line_edit)
        rename.addAction(rename_action)

    def __init_type_menu__(self):
        if self.value.type == RoiParameters.roi_types.ring:
            new_type = RoiParameters.roi_types.segment
            change_type_name = 'segment'
        else:
            new_type = RoiParameters.roi_types.ring
            change_type_name = 'ring'
        new_value = self.value._replace(type=new_type)
        change_type_action = self.addAction(f'Change type to {change_type_name}')
        change_type_action.triggered.connect(
            lambda: self.send(('change_roi_type', new_value)))


class RadialProfileContextMenu(RoiContextMenu):
    def __init_menu__(self):
        self.__init_rename_menu__()
        self.addSeparator()
        self.__init_type_menu__()
        self.addSeparator()
        self.__init_fix_menu__()
        self.addSeparator()
        self.__init_select_menu__()
        self.addSeparator()
        self.__init_fit_menu__()
        self.addSeparator()
        self.__init_delete_menu__()

    def __init_fit_menu__(self):
        fit_menu = self.addMenu('Fit')
        fit_selected = fit_menu.addAction('Fit selected (separately)')
        fit_selected.triggered.connect(lambda: self.send('fit_selected'))
        fit_together = fit_menu.addAction('Fit selected (together)')
        fit_together.triggered.connect(lambda: self.send('fit_together'))


# -*- coding: utf-8 -*-

from PyQt5.QtGui import QColor

# TODO: wrap into class (save config to json) and provide option for customization and changing colors


COLOR_DICT = dict(
    active=QColor(255, 139, 66, 70), active_bright=QColor(255, 139, 66),
    inactive=QColor(0, 0, 255, 50), inactive_bright=QColor(0, 0, 255),
    fixed=QColor(0, 255, 0, 50), fixed_bright=QColor(0, 255, 0),
    fixed_active=QColor(255, 0, 255, 50), fixed_active_bright=QColor(255, 0, 255)
)




from PyQt5.QtWidgets import (QLabel, QComboBox, QHBoxLayout)
from PyQt5.QtCore import Qt

from ..basic_widgets import (BasicInputParametersWidget,
                             AbstractInputParametersWidget)
from .modes import INTERPOLATION_MODES
from ...config import read_config
from ..basic_widgets import InfoButton


def get_interpolation_parameters(get_default_parameters: bool = False):
    return read_config(InterpolateSetupWindow.NAME, get_default_parameters)


class InterpolateSetupWindow(BasicInputParametersWidget):
    P = BasicInputParametersWidget.InputParameters

    # TODO: add info to interpolation parameters

    PARAMETER_TYPES = (P('r_size', 'Radius axis size', int),
                       P('phi_size', 'Angle axis size', int),
                       P('mode', 'Interpolation algorithm', str))

    NAME = 'Interpolation parameters'

    def _get_layout(self,
                    input_parameter: BasicInputParametersWidget.InputParameters):
        if input_parameter.name == 'mode':
            return self._get_mode_layout(input_parameter)
        else:
            return super()._get_layout(input_parameter)

    def _get_mode_layout(self, input_parameter: BasicInputParametersWidget.InputParameters):

        current_value = self.default_dict.get(input_parameter.name, None)
        if current_value is None:
            current_value = INTERPOLATION_MODES[0].name
        label_widget = QLabel(input_parameter.label)
        input_widget = QComboBox()
        input_widget.setEditable(False)
        input_widget.addItems([m.name for m in INTERPOLATION_MODES])
        input_widget.setCurrentText(current_value)
        layout = QHBoxLayout()
        layout.addWidget(label_widget, Qt.AlignHCenter)
        layout.addWidget(input_widget, Qt.AlignHCenter)
        if input_parameter.info:
            info_button = InfoButton(input_parameter.info)
            layout.addWidget(info_button, Qt.AlignLeft)

        def get_input(*args):
            return input_widget.currentText()

        setattr(AbstractInputParametersWidget, input_parameter.name, property(get_input))
        return layout






import logging
from collections import defaultdict

from pyqtgraph.dockarea import DockArea, Dock

from .signal_connection import AppDataHolder, AppNode, SignalContainer
from .control_widget import ControlWidget
from .plot_widgets import Basic2DImageWidget
from .interpolation.interpolation_widget import InterpolateImageWidget
from .radial_profile_widget import RadialProfileWidget
from .angular_profile_widget import AngularProfileWidget
from .file_manager import FileWidget

logger = logging.getLogger(__name__)


class AppDockArea(DockArea, AppNode):
    def __init__(self):
        DockArea.__init__(self)
        AppNode.__init__(self, AppDataHolder())
        self._status_dict = defaultdict(lambda: True)

        self.__init_image_view__()
        self.__init_control_widget__()
        self.__init_interpolate_widget__()
        self.__init_radial_widget__()
        self.__init_file_widget__()
        self.__init_angular_widget__()

        self._DOCK_DICT = {'interpolation': self.interpolation_dock,
                           'radial_profile': self.radial_profile_dock,
                           'control': self.control_dock,
                           'image_view': self.image_view_dock,
                           'file_widget': self.file_dock,
                           'angular_profile': self.angular_profile_dock}
        self.__apply_default_view__()

    def __apply_default_view__(self):
        self.show_hide_docks('interpolation')
        self.show_hide_docks('radial_profile')
        self.show_hide_docks('angular_profile')
        self.show_hide_docks('control')

    def __init_image_view__(self):
        self.image_view = Basic2DImageWidget(
            self.get_lower_connector('Basic2DImageWidget'))
        dock = Dock('Image')
        dock.addWidget(self.image_view)
        self.addDock(dock, size=(1000, 1000))
        self.image_view_dock = dock

    def __init_interpolate_widget__(self):
        self.interpolate_view = InterpolateImageWidget(
            self.get_lower_connector('InterpolateImageWidget'), self)
        dock = Dock('Interpolate')
        dock.addWidget(self.interpolate_view)
        self.addDock(dock, position='right')
        self.interpolate_view.update_image()
        self.interpolation_dock = dock

    def __init_radial_widget__(self):
        self.radial_profile = RadialProfileWidget(
            self.get_lower_connector('RadialProfileWidget'), self)
        dock = Dock('Radial Profile')
        dock.addWidget(self.radial_profile)
        self.addDock(dock, position='bottom')
        self.radial_profile.update_image()
        self.radial_profile_dock = dock

    def __init_angular_widget__(self):
        self.angular_profile = AngularProfileWidget(
            self.get_lower_connector('AngularProfileWidget'), self)
        dock = Dock('Angular Profile')
        dock.addWidget(self.angular_profile)
        self.addDock(dock, position='bottom')
        self.angular_profile.update_profile()
        self.angular_profile_dock = dock

    def __init_control_widget__(self):
        self.control_widget = ControlWidget(
            self.get_lower_connector('ControlWidget'), self)
        control_dock = Dock('Segments')
        control_dock.addWidget(self.control_widget)
        self.addDock(control_dock, position='right')
        self.control_dock = control_dock

    def __init_file_widget__(self):
        self.file_widget = FileWidget(self.get_lower_connector('FileWidget'), self)
        self.file_dock = Dock('Files')
        self.file_dock.addWidget(self.file_widget)
        self.addDock(self.file_dock, position='left')

    def update_plot(self, image):
        SignalContainer(app_node=self).data_changed(image).send()

    def show_hide_docks(self, dock_name: str):
        assert dock_name in self._DOCK_DICT.keys()
        dock = self._DOCK_DICT[dock_name]
        status = self._status_dict[dock_name]
        if status:
            dock.hide()
        else:
            dock.show()
        self._status_dict[dock_name] = not status

